#include "stdafx.h"
#include "Bar.h"
#include "IndicatorBarSource.h"
#include "CSVHistoryLoader.h"

#include <fstream>
#include <limits>
#include <sstream>
#include "Tokenizer.h"
#include "strptime.h"
#include <ctime>

CSVHistoryLoader::CSVHistoryLoader(void)
{
    mDateFrom = 0.0;
}

CSVHistoryLoader::~CSVHistoryLoader(void)
{
}

void CSVHistoryLoader::getHistoricalData(CIndicatorBarSource *source,
                                         DATE dateFrom,
                                         DATE dateTo)
{
    if (dateFrom < mDateFrom ||
        dateTo > mDateTo ||
        mInstrument != source->getInstrument() ||
        mTimeframe != source->getTimeframe())
        return;

    for (std::vector<Bar*>::iterator i = mBars.begin(); i != mBars.end(); ++i)
        if ((*i)->date >= dateFrom && (*i)->date <= mDateTo)
            source->addBar(*i);
}

bool CSVHistoryLoader::loadCSV(const char *fileName, bool bid)
{
    std::ifstream csv(fileName);
    if (!csv.is_open())
        return false;

    std::stringstream ss;
    std::string delimiters = ";";
    struct tm tmBuf;

    /** load header. */
    ss << csv;
    Tokenizer headerTokenizer(ss.str(), delimiters);

    /** check header tag. */
    if (!headerTokenizer.nextToken())
        return false;
    if (headerTokenizer.getToken() != "HDR")
        return false;

    /** get instrument. */
    if (!headerTokenizer.nextToken())
        return false;
    mInstrument = headerTokenizer.getToken();

    /** get dateFrom. */
    if (!headerTokenizer.nextToken())
        return false;
    strptime(headerTokenizer.getToken().c_str(), "%m.%d.%Y %H:%M:%S", &tmBuf);
    CO2GDateUtils::CTimeToOleTime(&tmBuf, &mDateFrom);

    /** get dateTo. */
    if (!headerTokenizer.nextToken())
        return false;
    strptime(headerTokenizer.getToken().c_str(), "%m.%d.%Y %H:%M:%S", &tmBuf);
    CO2GDateUtils::CTimeToOleTime(&tmBuf, &mDateTo);

    /** get timeframe. */
    if (!headerTokenizer.nextToken())
        return false;
    mTimeframe = headerTokenizer.getToken();

    while (!csv.eof())
    {
        ss.str("");
        ss << csv;

        Bar *bar = new Bar();

        Tokenizer tokenizer(ss.str(), delimiters);
        if (tokenizer.nextToken())
        {
            if (tokenizer.getToken() != "DAT")
                return false;
        }
        else
            continue;

        if (tokenizer.nextToken())
        {
            strptime(headerTokenizer.getToken().c_str(), "%m.%d.%Y %H:%M:%S", &tmBuf);
            CO2GDateUtils::CTimeToOleTime(&tmBuf, &(bar->date));
        }

        if (tokenizer.nextToken())
            if (bid)
                bar->date = atof(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (bid)
                bar->open = atof(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (bid)
                bar->high = atof(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (bid)
                bar->low = atof(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (bid)
                bar->close = atof(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (bid)
                bar->date = atof(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (!bid)
                bar->open = atof(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (!bid)
                bar->high = atof(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (!bid)
                bar->low = atof(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (!bid)
                bar->close = atof(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (bid)
                bar->date = atof(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (bid)
                bar->volume = atoi(tokenizer.getToken().c_str());
        if (tokenizer.nextToken())
            if (!bid)
                bar->volume = atoi(tokenizer.getToken().c_str());

        mBars.push_back(bar);

        mDateFrom = min(mDateFrom, bar->date);
        mDateTo = max(mDateTo, bar->date);
    }
    return true;
}

DATE CSVHistoryLoader::getDateFrom()
{
    return mDateFrom;
}

DATE CSVHistoryLoader::getDateTo()
{
    return mDateTo;
}


