#include "stdafx.h"
#include "Bar.h"
#include "IndicatorBarSource.h"
#include "HistoryLoader.h"
#include "CSVHistoryLoader.h"
#include "FXConnectHistoryLoader.h"

HistoryLoader::HistoryLoader (IInstance* instance,
                              int cookie,
                              CIndicatorBarSource* barSource,
                              DATE from,
                              DATE to,
                              IO2GSession *session,
                              std::string &csvSource,
                              HANDLE notifyFinished) : sample_tools::AThread()
{
    if (instance->getType() == IInstance::TIndicator)
        mIndicatorInstance = (IIndicatorInstance*)instance;
    else
        mIndicatorInstance = NULL;

    mCookie = cookie;
    mIndicatorBarSource = barSource;
    mDateFrom = from;
    mDateTo = to;
    mSession = session;
    if (mSession)
        mSession->addRef();

    mCSVSource = csvSource;
    mNotifyFinished = notifyFinished;
}

HistoryLoader::~HistoryLoader(void)
{
    if (mSession)
        mSession->release();
}

int HistoryLoader::run()
{
    if (!mCSVSource.empty())
    {
        CSVHistoryLoader *csvHistoryLoader = new CSVHistoryLoader();
        if (csvHistoryLoader->loadCSV(mCSVSource.c_str(), mIndicatorBarSource->isBid()))
        {
            if (csvHistoryLoader->getDateFrom() > mDateFrom)
            {
                FXConnectHistoryLoader *fxcHistoryLoader = new FXConnectHistoryLoader(mSession);
                fxcHistoryLoader->getHistoricalData(mIndicatorBarSource, mDateFrom, csvHistoryLoader->getDateFrom());
                mError = fxcHistoryLoader->getError();
                mDateFrom = csvHistoryLoader->getDateFrom();
            }
            DATE dtTo = mDateTo;
            if (csvHistoryLoader->getDateTo() < mDateTo || mDateTo == 0.0)
                dtTo = csvHistoryLoader->getDateTo();
            csvHistoryLoader->getHistoricalData(mIndicatorBarSource, mDateFrom, dtTo);
            mDateFrom = dtTo;
        }
    }

    if (mSession && (mDateFrom < mDateTo || mDateTo == 0.0))
    {
        /** Should check what was loaded by CSVHistoryLoader and load missing data. */
        FXConnectHistoryLoader *fxcHistoryLoader = new FXConnectHistoryLoader(mSession);
        fxcHistoryLoader->getHistoricalData(mIndicatorBarSource, mDateFrom, mDateTo);
        mError = fxcHistoryLoader->getError();
    }

    mIndicatorInstance->notifyFinished(mCookie, mError.empty(), NULL, NULL);
    if (mNotifyFinished != INVALID_HANDLE_VALUE)
        SetEvent(mNotifyFinished);

    return 0;
}

const char *HistoryLoader::getError()
{
    return mError.c_str();
}