#include "StdAfx.h"
#include "Bar.h"
#include "IndicatorBarSource.h"
#include <cmath>

CIndicatorBarSource::CIndicatorBarSource(const char *timeframe, const char *instrument, double pipSize, int precision, bool isBid)
{
    mTimeframe = timeframe;
    mInstrument = instrument;
    mPipSize = pipSize;
    mPrecision = precision;
    mSerial = 0;
    mIsBid = isBid;
}

const char *CIndicatorBarSource::getName()
{
    return "CIndicatorBarSource";
}

int CIndicatorBarSource::size()
{
    return (int)mBars.size();
}

bool CIndicatorBarSource::isBar()
{
    return true;
}

bool CIndicatorBarSource::hasData(int index)
{
    if (index >= 0 && index < (int)mBars.size())
        return true;
    else
        return false;
}

bool CIndicatorBarSource::supportsVolume()
{
    return true;
}

int CIndicatorBarSource::getFirstPeriod()
{
    return 0;
}

double CIndicatorBarSource::date(int index)
{
    return mBars[index].bar->date;
}

double CIndicatorBarSource::getDate(int index)
{
    return mBars[index].bar->date;
}


int CIndicatorBarSource::getSerial(int index)
{
    return mBars[index].serial;
}

const char *CIndicatorBarSource::getInstrument()
{
    return mInstrument.c_str();
}
void CIndicatorBarSource::setInstrument(const char *instrument)
{
    mInstrument = instrument;
}

const char *CIndicatorBarSource::getTimeframe()
{
    return mTimeframe.c_str();
}

double CIndicatorBarSource::getPipSize()
{
    return mPipSize;
}

void CIndicatorBarSource::setPipSize(double pipSize)
{
    mPipSize = pipSize;
}

bool CIndicatorBarSource::isBid()
{
    return mIsBid;
}

bool CIndicatorBarSource::isAlive()
{
    return true;
}

int CIndicatorBarSource::getPrecision()
{
    return mPrecision;
}

void CIndicatorBarSource::setPrecision(int precision)
{
    mPrecision = precision;
}

int CIndicatorBarSource::getDisplayPrecision()
{
    return mPrecision;
}

double CIndicatorBarSource::getOpen(int index)
{
    return mBars[index].bar->open;
}

double CIndicatorBarSource::getHigh(int index)
{
    return mBars[index].bar->high;
}

double CIndicatorBarSource::getLow(int index)
{
    return mBars[index].bar->low;
}

double CIndicatorBarSource::getClose(int index)
{
    return mBars[index].bar->close;
}

double CIndicatorBarSource::getVolume(int index)
{
    return mBars[index].bar->volume;
}

const char *CIndicatorBarSource::getBarSize(void)
{
    return mTimeframe.c_str();
}

void CIndicatorBarSource::release()
{
    std::cout << "BarSource deleted" << std::endl;
    delete this;
}

bool CIndicatorBarSource::getOHLC(int index, double &open, double &high, double &low, double &close)
{
    if (index >= 0 && index < (int)mBars.size())
    {
        Bar *b = mBars[index].bar;
        open = b->open;
        high = b->high;
        low = b->low;
        close = b->close;
        return true;
    }
    else
        return false;
}


void CIndicatorBarSource::updateBar(int index, double open, double high, double low, double close, int volumeDiff)
{
    sample_tools::Mutex::Lock l(&mMutex);

    if (index >= 0 && index < (int)mBars.size())
    {
        Bar *b = mBars[index].bar;

        b->high = max(b->high, high);
        b->low = min(b->low, low);
        b->close = close;

        b->volume += volumeDiff;
    }
}

void CIndicatorBarSource::addBar(Bar* bar)
{
    sample_tools::Mutex::Lock l(&mMutex);

    mBars.push_back(IndicatorBar(bar, ++mSerial));
}

CIndicatorBarSource *CIndicatorBarSource::getSlice(DATE from, DATE to)
{
    sample_tools::Mutex::Lock l(&mMutex);

    std::vector<IndicatorBar>::iterator it = mBars.begin();
    if (from < mBars[0].bar->date)
    {
        while (it != mBars.end() && it->bar->date > from)
            ++it;
    }

    if (it == mBars.end())
        return NULL;

    CIndicatorBarSource *barSource = new CIndicatorBarSource(mTimeframe.c_str(), mInstrument.c_str(), mPipSize, mPrecision, mIsBid);

    while (it != mBars.end() && it->bar->date > from)
    {
        barSource->addBar(new Bar(*(it->bar)));
        ++it;
    }

    return barSource;
}