#include "StdAfx.h"
#include "Bar.h"
#include "IndicatorBarSource.h"
#include "ResponseListener.h"
#include "LiveTicksUpdater.h"

LiveTicksUpdater::LiveTicksUpdater(IO2GSession *session,
                                   int tradingDayOffset,
                                   int tradingWeekOffset)
{
    mSession = session;
    if (mSession)
        mSession->addRef();
    mTradingDayOffset = tradingDayOffset;
    mTradingWeekOffset = tradingWeekOffset;
    mBarConstructed = NULL;
    mLastBarMinuteVolume = 0;
    mLastBarUpdateMinute = 0;
}

LiveTicksUpdater::~LiveTicksUpdater(void)
{
    if (mSession)
        mSession->release();
}

void LiveTicksUpdater::subscribeOfferUpdates(IIndicatorInstance *indicatorInst,
                                             CIndicatorBarSource *source)
{
    mIndicatorInstance = indicatorInst;
    mSource = source;

    if (mBarConstructed)
    {
        delete mBarConstructed;
        mBarConstructed = NULL;
    }
    CandlePeriod::parsePeriod(source->getTimeframe(), mBarUnit, mBarUnitLength);

    mResponseListener = new ResponseListener(mSession);
    mResponseListener->setTicksUpdater(this);
    mSession->subscribeResponse(mResponseListener);
}

void LiveTicksUpdater::unsubscribeUpdates()
{
    sample_tools::Mutex::Lock l(&mMutex);

    mSession->unsubscribeResponse(mResponseListener);
}

inline DATE __roundToMinutes(const DATE d)
{
    SYSTEMTIME st;
    DATE dRes = 0.0;
    CO2GDateUtils::OleTimeToWindowsTime(d, &st);
    st.wSecond = 0;
    st.wMilliseconds = 0;
    CO2GDateUtils::WindowsTimeToOleTime(&st, &dRes);
    return dRes;
}

void LiveTicksUpdater::onNewTick(IO2GOfferRow *offer)
{
    if (!offer)
        return;

    if (strcmp(offer->getInstrument(), mSource->getInstrument()) == 0)
    {
        sample_tools::Mutex::Lock l(&mMutex);

        if (!mBarConstructed)
            mBarConstructed = new Bar();

        DATE tickCandleStart; 
        DATE tickCandleEnd; 
        CandlePeriod::getCandle(offer->getTime(), tickCandleStart, tickCandleEnd, mBarUnit, mBarUnitLength, mTradingDayOffset, mTradingWeekOffset);

        if (tickCandleStart > mBarConstructed->date)
        {
            mSource->addBar(mBarConstructed);

            const char *err;
            mIndicatorInstance->update(true, &err);

            mBarConstructed = new Bar();
            mBarConstructed->volume = 0;
            if (mSource->isBid())
                mBarConstructed->open = offer->getBid();
            else
                mBarConstructed->open = offer->getAsk();
        }
        
        mBarConstructed->date = tickCandleStart;
        if (__roundToMinutes(offer->getTime()) > mLastBarUpdateMinute)
            mLastBarMinuteVolume = 0;

        mBarConstructed->volume += offer->getVolume() - mLastBarMinuteVolume;

        mLastBarMinuteVolume = offer->getVolume();

        if (mSource->isBid())
        {
            mBarConstructed->close = offer->getBid();
            mBarConstructed->high = max(mBarConstructed->high, offer->getBid());
            mBarConstructed->low = min(mBarConstructed->low, offer->getBid());
        }
        else
        {
            mBarConstructed->close = offer->getAsk();
            mBarConstructed->high = max(mBarConstructed->high, offer->getAsk());
            mBarConstructed->low = min(mBarConstructed->low, offer->getAsk());
        }

        mLastBarUpdateMinute = __roundToMinutes(offer->getTime());
    }
}