#include "stdafx.h"
#include <string>
#include <cstring>
#include "LoginParams.h"

const char *LoginParams::Strings::loginNotSpecified = "'Login' is not specified (/l|-l|/login|--login)";
const char *LoginParams::Strings::passwordNotSpecified = "'Password' is not specified (/p|-p|/password|--password)";
const char *LoginParams::Strings::urlNotSpecified = "'URL' is not specified (/u|-u|/url|--url)";
const char *LoginParams::Strings::connectionNotSpecified = "'Connection' is not specified (/c|-c|/connection|--connection)";

LoginParams::LoginParams(int argc, char **argv)
{
    /* Load parameters with short keys. */
    mLogin = getRequiredArgument(argc, argv, "l");
    mPassword = getRequiredArgument(argc, argv, "p");
    mURL = getRequiredArgument(argc, argv, "u");
    mConnection = getRequiredArgument(argc, argv, "c");

    /* If parameters with short keys not loaded, load with long keys. */
    if (mLogin.empty())
        mLogin = getRequiredArgument(argc, argv, "login");
    if (mPassword.empty())
        mPassword = getRequiredArgument(argc, argv, "password");
    if (mURL.empty())
        mURL = getRequiredArgument(argc, argv, "url");
    if (mConnection.empty())
        mConnection = getRequiredArgument(argc, argv, "connection");

    /* Load optional parameters. */
    mSessionID = getRequiredArgument(argc, argv, "subsession");
    mPin = getRequiredArgument(argc, argv, "pin");

    /* Append "Hosts.jsp" to URL if needed. */
    if (mURL.length() != 0)
    {
        // TODO: case insensitive find.
        if (mURL.find("Hosts.jsp") == std::string::npos)
        {
            mURL += "/Hosts.jsp";
        }
    }
}

LoginParams::~LoginParams(void)
{
}

const char *LoginParams::getRequiredArgument(int argc, char **argv, const char *key)
{
    for (int i = 1; i < argc; ++i)
    {
        if (argv[i][0] == '-' || argv[i][0] == '/')
        {
            int delimOffset = 1;
            if (strncmp(argv[i], "--", 2) == 0)
                ++delimOffset;

            if (_stricmp(argv[i] + delimOffset, key) == 0)
            {
                ++i;
                return argv[i];
            }
        }
    }
    return "";
}

const char * LoginParams::getLogin()
{
    return mLogin.c_str();
}

const char * LoginParams::getPassword()
{
    return mPassword.c_str();
}

const char * LoginParams::getURL()
{
    return mURL.c_str();
}

const char * LoginParams::getConnection()
{
    return mConnection.c_str();
}

const char * LoginParams::getSessionID()
{
    return mSessionID.c_str();
}

const char * LoginParams::getPin()
{
    return mPin.c_str();
}
