#include "StdAfx.h"
#include "Bar.h"
#include "IndicatorBarSource.h"
#include "LiveTicksUpdater.h"
#include "ResponseListener.h"

ResponseListener::ResponseListener(IO2GSession *session)
{
    mSession = session;
    if (mSession)
        mSession->addRef();
    mRefCount = 1;
    mRequestEvent = CreateEvent(0, FALSE, FALSE, 0);
    mRequestID = "";
    mLastResponse = NULL;
    mLiveTicksUpdater = NULL;
    mError = "";
    mHasError = false;
}

ResponseListener::~ResponseListener()
{
    if (mLastResponse)
        mLastResponse->release();
    if (mSession)
        mSession->release();
    CloseHandle(mRequestEvent);
}

HANDLE ResponseListener::getRequestEvent()
{
    return mRequestEvent;
}

IO2GResponse *ResponseListener::getLastResponse()
{
    if (mLastResponse)
    {
        mLastResponse->addRef();
        return mLastResponse;
    }
    return NULL;
}

bool ResponseListener::hasError()
{
    return mHasError;
}

const char *ResponseListener::getError()
{
    return mError.c_str();
}

/** Increase reference counter. */
long ResponseListener::addRef()
{
    return InterlockedIncrement(&mRefCount);
}

/** Decrease reference counter. */
long ResponseListener::release()
{
    long rc = InterlockedDecrement(&mRefCount);
    if (rc == 0)
        delete this;
    return rc;
}

/** Set request. */
void ResponseListener::setRequest(const char *requestId)
{
    mRequestID = requestId;
    mError.clear();
    mHasError = false;
    if (mLastResponse)
    {
        mLastResponse->release();
        mLastResponse = NULL;
    }
    ResetEvent(mRequestEvent);
}

void ResponseListener::setTicksUpdater(LiveTicksUpdater *updater)
{
    mLiveTicksUpdater = updater;
}

/** Request execution completed data handler. */
void ResponseListener::onRequestCompleted(const char *requestId, IO2GResponse  *response)
{
    if (response && mRequestID == requestId)
    {
        mLastResponse = response;
        mLastResponse->addRef();
        SetEvent(mRequestEvent);
    }
}

/** Request execution failed data handler. */
void ResponseListener::onRequestFailed(const char *requestId , const char *error)
{
    if (mRequestID == requestId)
    {
        mError = error;
        mHasError = true;
        SetEvent(mRequestEvent);
    }
}

/** Request update data received data handler. */
void ResponseListener::onTablesUpdates(IO2GResponse *data)
{
    if (data && data->getType() == TablesUpdates)
    {
        O2G2Ptr<IO2GResponseReaderFactory> readerFactory = mSession->getResponseReaderFactory();
        if (readerFactory)
        {
            O2G2Ptr<IO2GTablesUpdatesReader> reader = readerFactory->createTablesUpdatesReader(data);
            if (reader)
                for (int i = 0; i < reader->size(); ++i)
                    if (mLiveTicksUpdater)
                        mLiveTicksUpdater->onNewTick(reader->getOfferRow(i));
        }
    }
}
