#include "stdafx.h"
#include <string>
#include <cstring>
#include "SampleParams.h"
#include "strptime.h"

const char *SampleParams::Strings::instrumentNotSpecified = "'Instrument' is not specified (/i|-i|/instrument|--instrument)";
const char *SampleParams::Strings::timeframeNotSpecified = "'Timeframe' is not specified (/timeframe|--timeframe)";
const char *SampleParams::Strings::dateFromNotSpecified = "'DateFrom' is not specified (/datefrom|--datefrom)";
const char *SampleParams::Strings::dateToNotSpecified = "'DateTo' is not specified (/dateto|--dateto)";
const char *SampleParams::Strings::indicatorsPathNotSpecified = "'IndicatorsPath' is not specified (/indicators|--indicators)";
const char *SampleParams::Strings::csvSourceNotSpecified = "'CSVSource' is not specified (/csv|--csv)";

SampleParams::SampleParams(int argc, char **argv)
{
    /* Load parameters with short keys. */
    mInstrument = getRequiredArgument(argc, argv, "i");

    /* If parameters with short keys not loaded, load with long keys. */
    if (mInstrument.empty())
        mInstrument = getRequiredArgument(argc, argv, "instrument");

    /* Load parameters with long keys. */
    mTimeframe = getRequiredArgument(argc, argv, "timeframe");
    mIndicatorsPath = getRequiredArgument(argc, argv, "indicators");
    mCSVSource = getRequiredArgument(argc, argv, "csv");

    std::string sDateFrom = getRequiredArgument(argc, argv, "datefrom");
    std::string sDateTo = getRequiredArgument(argc, argv, "dateto");
    std::string sIsBid = getRequiredArgument(argc, argv, "bid", true);

    /* Convert types. */
    struct tm tmBuf = {0};

    if (sDateFrom.empty())
        mDateFrom = 0.0;
    else
    {
        strptime(sDateFrom.c_str(), "%m.%d.%Y %H:%M:%S", &tmBuf);
        CO2GDateUtils::CTimeToOleTime(&tmBuf, &mDateFrom);
    }

    if (sDateTo.empty())
        mDateTo = 0.0;
    else
    {
        strptime(sDateTo.c_str(), "%m.%d.%Y %H:%M:%S", &tmBuf);
        CO2GDateUtils::CTimeToOleTime(&tmBuf, &mDateTo);
    }

    mIsBid = (!sIsBid.empty());
}

SampleParams::~SampleParams(void)
{
}

const char *SampleParams::getRequiredArgument(int argc, char **argv, const char *key, bool isBoolean)
{
    for (int i = 1; i < argc; ++i)
    {
        if (argv[i][0] == '-' || argv[i][0] == '/')
        {
            int delimOffset = 1;
            if (strncmp(argv[i], "--", 2) == 0)
                ++delimOffset;

            if (_stricmp(argv[i] + delimOffset, key) == 0)
            {
                if (isBoolean)
                    return "true";
                else
                {
                    ++i;
                    return argv[i];
                }
            }
        }
    }
    return "";
}

/** Getters. */

const char *SampleParams::getInstrument()
{
    return mInstrument.c_str();
}

const char *SampleParams::getTimeframe()
{
    return mTimeframe.c_str();
}

DATE SampleParams::getDateFrom()
{
    return mDateFrom;
}

DATE SampleParams::getDateTo()
{
    return mDateTo;
}

const char *SampleParams::getIndicatorsPath()
{
    return mIndicatorsPath.c_str();
}

const char *SampleParams::getCSVSource()
{
    return mCSVSource.c_str();
}

bool SampleParams::isBid()
{
    return mIsBid;
}